//
//| 1F_EViewsMT4.mq4                                                                               |
//| Expert Advisor based on forecasts obtained from the EViews application package                                    |  
//|   Version of:  2011.09.28                                                                       |         
//
//|   The Expert Advisor trades short and long on M1 time frame. A forecast step in minutes is specified in the parameters      |
//|   Algorithm:                                                                                    |
//|      every forecast step StepForecast(multiple of the forecast step)                                   |
//|         the Expert Advisor generates files from the NumberBars bars for EViews                                |
//|      after the files are saved, EViews is started                                                   |
//|      Eviews calculates the forecast and indicates forecast errors for every time frame;             |
//|      EViews generates the file with results in the 4 folder                                         |
//|      The Expert Advisor gets the forecast file as soon as its generation is completed                         |
//|      Decisions on positions are taken.                                                      |
//|   The position is entered if the position is not present                                      |
//|   The Expert Advisor exits positions if the following conditions are met:                          |
//|      if the forecast changes to its opposite                                                 |
//|      if the take profit signal is received upon reaching the upper limit of the forecast channel   |
//|      if the stop loss signal is received upon reaching the lower limit of the forecast channel    |
//
//
#property copyright "One TF ahead forecast"
#property link      "faa1947@mail.ru"
//
//--------------- Include standard libraries---------------------------------------------------
#include <stdlib.mqh>                  //
#include <stderror.mqh>                // Constants for error processing
#include <WinUser32.mqh>               // Windows constants 
//
// ---------------- API function library for calling EViews --------------------------------------
#import  "kernel32.dll"
   int      WinExec(string NameEx, int dwFlags);
#import
//
//-----------------External variables -------------------------------------------------------------
extern   int      StepForecast            =  PERIOD_H1;        // One-step-ahead forecast in minutes
extern   int      NumberBars              =  101;              // Number of bars for the forecast calculation
extern   double   MultSE                  =  2;                // Number of standard deviations
//
//-----------------Global variables ----------------------------------------------------------
         int      ErStop                  =  0;                // Stop of operation of the Expert Advisor
         double   Forecast;
         double   Forecast_SE;
         double   Trend;                                       // New trend
         double   TrendPrev;                                   // Previous trend
         bool     FirstForecast           =  true;             // Marker of the first forecast

         double   CommonSL                =  0;
         double   CommonTP                =  0;
         double   slip;          
         int      NumberForecast          =  0;                // Number of forecasts for the number of the line 
                                                               // Profit table in EViews
         int      GetError;//
         int      CountLabel;
//  Main function Start 
int start()                               
{
//
// ---------------- Local variables ----------------------------------------------------------
   static   int         prevTF      =  0;
   static   int         prevTF_M1      =  0;
//
// ------------------ Calculate once on M1 ------------------------------------------------------
   if (ErStop                    != 0)
   {
      Print("");
      Print("E x p e r t    a d v i s o r    e m e r g e n c y    s t o p ");
      Print("");      
      return;
   }  
  
   if (iTime(NULL, PERIOD_M1, 1) != prevTF_M1)
   {
      prevTF_M1                  =  iTime(NULL, PERIOD_M1, 1);

      if(Trend                   <  0.0)
      {
         DrawArrow(-Forecast,  119);
         DrawArrow(-(Forecast  +  Forecast_SE),  160);
         DrawArrow(-(Forecast  -  Forecast_SE),  160);  
//         DrawArrow(-(Forecast  +  2*Forecast_SE),  160);
//         DrawArrow(-(Forecast  -  2*Forecast_SE),  160);                  
      }
      else
      {
         DrawArrow(Forecast,  119); 
         DrawArrow(Forecast  +  Forecast_SE,  160);
         DrawArrow(Forecast  -  Forecast_SE,  160);                 
//         DrawArrow(Forecast  +  2*Forecast_SE,  160);
//         DrawArrow(Forecast  -  2*Forecast_SE,  160);       
      }
// --------      
      return;
   }
// Time Frame 
   if (iTime(NULL, StepForecast, 1)    != prevTF)           // Waiting for the time frame
   {
      prevTF                     =  iTime(NULL, StepForecast, 1); 
      ReadEViews();
      Enter_1();

      return;      
   }                                            
// -------------------------------------------------------------------------------------------------
          
   return;                             
}        
// --------------------------Exit the Start() - exit the Expert Advisor ----------------------------------
//
// Expert Advisor initialization  
int init()                           
{
   int   lastError; 

  // ------ Check the validity of the forecast step ----------------------------------------------------
   switch(StepForecast)
   {
      case PERIOD_M5:
      case PERIOD_M15:
      case PERIOD_M30:
      case PERIOD_H1:
      case PERIOD_H4:
      case PERIOD_D1:
      case PERIOD_W1:
         break;
      default:
         ErStop                  =  1;

      return;                                   // Expert Advisor emergency blocking
   }
/*
   FileDelete("EViewsForecast.txt");
   lastError               = GetLastError();
   if(lastError            > 1     &&
      lastError            != 4100)             // 4100 means that there is no file
   {
      Print("Error (",lastError,") deleting EViewsForecast.txt");
   }
*/
   FileDelete("KOTIR.txt");
   lastError                = GetLastError();
   if(lastError            > 1     &&
      lastError            != 4100)             // 4100 means that there is no file
   {
      Print("Error (",lastError,") deleting KOTIR.txt");
   }

   FileDelete("work.wf1");
   lastError                = GetLastError();
   if(lastError            > 1     &&
      lastError            != 4100)             // 4100 means that there is no file
   {
      Print("Error (",lastError,") deleting work.wf1");
   }
}                                    //End of initialization function

//  deinit() 
int deinit()                           
{
   Print(" "); 
   Print("deinit - completion of operation of the Expert Advisor");
   
   if(ErStop            == 1)
   {
      Print("Error of the forecast step - StepForecast = ", StepForecast);
      Print("StepForecast shall be equal to the standard time frame in minutes");
   }
      
   if(OrdersTotal()     >  0)
   {
      Print("Number of open orders = ",    OrdersTotal());
   }
   return(0);
}
//  End of deinit() function    
//
//  Internal functions 
//
// ----------------- Function of data exchange between 4 and EViews ------------------------------------                                                        
int  ReadEViews()
{
// ------------------- Internal variables ----------------------------------   
   string   file_kotir;                         // 1 quotes file

   int      handle;                             // Pointer to the file
   string   fileForecast   =  "EViewsForecast.txt";
   
   int      i;
//---
// -------------- Generating file names for EViews --------------------------
   file_kotir              =  "KOTIR.txt";      // H1 quotes file name
// --------------- Creating the quotes file ------------------------------------
  
   handle                  = FileOpen(file_kotir,FILE_CSV|FILE_WRITE,',');
   if(handle               < 1)
   {
      Print("Failed to create the file ", 
            file_kotir,
            ". Error #", GetLastError());
      return(0);                                // Emergency shut-down
    //FileClose(handle);
   }
// ------------- Writing quotes --------------------------------------------
   FileWrite(handle, "DATE",   "kotir");        // header
   for (i   =  NumberBars  -  1; i  >= 0; i--)
   {
      FileWrite(handle,  TimeToStr(iTime(Symbol(),StepForecast,i)), 
                                    iOpen(Symbol(),StepForecast,i));
   }

   FileWrite(handle, "Forecast ", 0);             // Forecast area
   FileClose(handle);                            // Close the file

   NumberForecast ++;

   Print("");   
   Print("Created KOTIR.txt with the number of observations = ", 
            NumberBars+1);
   Print("Last values: date  = ",  TimeToStr(iTime(Symbol(),StepForecast,0)),
         ", n = ",                     iOpen(Symbol(),StepForecast,0));             
// ------------------- Reading the forecast from EViews -------------------------------
   if(IsOptimization()     ||
      IsTesting()          ||
      IsVisualMode())
   {
      string   W_E         =  "G:\Program Files\EViews7\EViews7.exe G:\ProgramEViews\mod2_t";
      WinExec(W_E, 0);                          // "0" means the start with closed windows

      handle               =  FileOpen(fileForecast,FILE_CSV|FILE_READ,',');
      i                    =  0;
      while(handle         <  0)                // Waiting for the forecast file to open
      {
         handle            =  FileOpen(fileForecast,FILE_CSV|FILE_READ,',');  
         i                 =  i  +  1; 
         if(i              >  1000)
         {
            Print("Attempts to open EViewsForecast.txt exceeded 1000.");
            Print("Aborting attempts to open the file.");
            break; 
         }
      }      
      Print("The number of attempts to open the forecast file = ", i);
   }
   else
   {
      WinExec("G:\Program Files\EViews7\EViews7.exe G:\ProgramEViews\mod2_t", 1);
                       // With an open window
      Sleep(10000);                          // Waiting for 10 seconds                 
      handle               =  FileOpen(fileForecast,FILE_CSV|FILE_READ,',');
      i                    =  0;
      while(handle         <  0)                // Waiting for the forecast file to open
      {
         handle            =  FileOpen(fileForecast,FILE_CSV|FILE_READ,',');  
         i                 =  i  +  1;               
         if(i              >  1000)
         {
            Print("Attempts to open EViewsForecast.txt exceeded 1000.");
            Print("Aborting attempts to open the file.");
            break; 
         }
      }      
      Print("The number of attempts to open the forecast file = ", i);
   }
//
// ------------------- Reading the forecast file  ----------------------------------
   FileReadNumber(handle);                      // Reading the table header
   FileReadNumber(handle);
   
   FileReadNumber(handle);                      // Reading the observation number
   Forecast                = FileReadNumber(handle);
   FileReadNumber(handle);                      // Reading the observation number
   Forecast_SE             = FileReadNumber(handle);
   FileReadNumber(handle);                      // Reading the observation number
   Trend                   = -FileReadNumber(handle);
      
   if(handle               <  0)
   {
      Print("Failed to read the file ",  
            fileForecast, 
            ", will retry in a minute. Error #", 
            GetLastError());
      return(-1);
   }   
//   Print("Successfully read the file ",  
//         fileForecast);               
// ------------------  Determining trends ---------------------------------------        
// -------------------- Determining trend direction --------------------------
// ----------(for use see Enter_1)
   if(FirstForecast)
   {
      TrendPrev            =  Trend;            // Previous value of the trend
      FirstForecast        =  false;
   } 
   Print("");
   Print("TrendPrev = ",         TrendPrev); 
   Print("Trend     = ",         Trend,
         ", Open = ",            Open[0],
         ",  Forecast = ",       Forecast,
         ", Forecast_SE = ",     Forecast_SE);  
   FileClose(handle);                 

// -------------
   return(0);
}

//-------------End of the forecast reading function ----------------------------------
//
//
// --------------------- Function of entry to a position ------------------------------
void   Enter_1()                              
{
// ------------------- Internal variables ----------------------------------   
   double   rr;
// -----------------  Closing upon change in the trend ----------------------------
/* --- Blocking exit from the position upon continuing trend
   if(TrendPrev *  Trend          <  0.0      &&
      OrdersTotal()              >  0)
   {
      Print("E-1: closing ",      OrdersTotal(),   
            " the position: the previous and the current trend forecasts do not coincide");
      ClosePozition(0);                         // Close all positions of all types            
   }
   TrendPrev                         =  Trend;

   if(OrdersTotal()              >  0)          // Do not add to the position
      return;
*/      
   if(OrdersTotal()        >  0)                // Do not add to the position
      ClosePozition(0);                         // Close all positions of all types            

// ------------------ Entry no. 1 to a short position ------------------------------------------
   if(Trend                 >  0.0)
   {
// ---------- Setting SL for a short position -----------------------------------------
      if(Forecast_SE - Ask >= 0.0)
         CommonSL          =  Ask   +  Forecast_SE *  MultSE ;
      else
         CommonSL          =  Ask   +  slip;
         
         CommonSL          =  Ask   +  0.0100;    // Without stop - temporarily
         
// ---------- Setting  for a short position -----------------------------------------
      if(Forecast_SE - Bid >= 0.0)
         CommonTP          =  Bid   -  Forecast_SE *  MultSE ;
      else
         CommonTP          =  Bid   -  slip;  
              
         CommonTP          =  Bid   -  0.0100;  // Without take profit - temporarily

         Open_Ord(1); 

      return;
   }
// ------------------ Entry no. 1 to a long position ------------------------------------------
   if(Trend                   <  0.0)   
   {
// ---------- Setting SL for a long position -----------------------------------------
      if(Forecast_SE - Bid >= 0.0)
         CommonSL          =  Bid -  Forecast_SE   *  MultSE ;
      else
         CommonSL          =  Bid   -  slip;
         
         CommonSL          =  Bid   -  0.0100;  // Without stop - temporarily
         
// ---------- Setting  for a long position -----------------------------------------
      if(Forecast_SE - Ask >= 0.0)
         CommonTP          =  Ask   +  Forecast_SE *  MultSE ;
      else
         CommonTP          =  Ask   +  slip;   
         
         CommonTP          =  Ask   +  0.0100;   // Without take profit - temporarily
            
         Open_Ord(-1); 

      return;
   }
}
//-------------End of function of entry to a position  ---------------------------------

//
//|   ClosePozition.mqh                                                       |
// ----------------------------------------------------------------------------
int ClosePozition(int NumberEnter)              // User function
{
   int      Tip,  err;                          // Order type
   int      Ticket;                    
   int      Total;                              // Number of orders
   int      i,    ii;
   double   Lot      =  0.0;                    // Number of closed lots
   double   Price_Cls,                          // Closing price of the order
            Profit;
   int      NumberOfPoz,                        // Number of closed positions
            magic,                              // Entry type identifier
            slip     =  3,                      //  slippage at the closing    
            freez; 
   string   Symb        =  Symbol();            
   string   comm,
            Text;
// ----------------------------------------------------------------------------

   Total             =  OrdersTotal();          // Number of orders
   if(Total          == 0)
      {
         Print("No orders for closing");
         return(0);
      }
   NumberOfPoz       =  0;
// --------- Waiting while the market is busy ----------------------------------------
   while (!IsTradeAllowed())
   { 
      Sleep(5000);
   }
   if(NumberEnter          <  0)
      Text                 =  " long positions";
   else
      Text                 =  " short positions";   
// --------- Closing all orders -----------------------------------------------
   for(i =  Total -  1; i  >= 0;   i--)         
   {
      if(OrderSelect(i,   SELECT_BY_POS))
      {
         Tip               =  OrderType();
         Ticket            =  OrderTicket();
         Lot               =  OrderLots();
         Lot               =  NormalizeDouble(Lot,  1); 
         comm              =  OrderComment();
         Profit            =  OrderProfit();         
         magic             =  OrderMagicNumber();

         if(Tip            == OP_BUY)
         {
            if(NumberEnter    <  -99            ||    // Close all long positions
               magic          == NumberEnter    ||    // Close long positions of this type
        //       Profit         <  0.0            ||
               NumberEnter    == 0)                   // Close all positions
            {

               RefreshRates(); 
               Price_Cls      =  NormalizeDouble(Bid,  Digits); 

               Print("");
               Print("Closing ", comm);

               if(!OrderClose(Ticket,   Lot,  Price_Cls,  MarketInfo(Symb,MODE_STOPLEVEL), Aqua))
               {                  

                  Print("Closing error. ",  comm,
                        ", price = ",     Price_Cls,
                        ", slippage =",   MarketInfo(Symb,MODE_STOPLEVEL) );
                  continue;
               }
               else
               {
                  Print("Successfully closed ", comm);
                  NumberOfPoz++;
               }   
               continue;                    
            }
         }      
         if(Tip               == OP_SELL)
         {
            if(NumberEnter    >  99             ||    // Close all short positions
               magic          == NumberEnter    ||    // Close short positions of this type
//               Profit         <  0.0            ||
               NumberEnter    == 0)                   // Close all positions
            {

               RefreshRates(); 
               Price_Cls      =  NormalizeDouble(Ask,  Digits); 
               
               Print("");
               Print("Closing ", comm);

               if(!OrderClose(Ticket,   Lot,  Price_Cls,  MarketInfo(Symb,MODE_STOPLEVEL), Red))
               {                  
                  Print("Closing error. ",  comm,
                        ", price = ",     Price_Cls,
                        ", slippage =",   MarketInfo(Symb,MODE_STOPLEVEL) );
                  continue;
               }
               else
               {
                  NumberOfPoz++;
                  Print("Successfully closed ", comm);
               }
               continue;                     
            }
         }
      }
   } 
   return(0);                                   //All orders processed
}
//
//
// -------------------- Order opening function --------------------------------
int Open_Ord(int  op)
{
   int      Ticket,                             // Order number
            errors,
            StopLevel,
            i;
   int      MagicNum;                           // MagicNumber
   double   slip;                               // Max deviation = spread
            
   double   r_lots,                             // Number of lots
            price;
   string   sy       =  Symbol(); 
   int      Total    =  OrdersTotal(),          // Number of orders
            slippage =  MarketInfo(Symbol(),MODE_SPREAD)   +  1;
   string   comm,
            TextOp,
            TextAccount,
            Enter,
            TextEnter,
            TextPrice,
            TextTime; 
// ------------------ Generating text for comments ------------------------
   MagicNum                   =  op;   

   if(op                      >  0)
   { 
      TextOp                  =  "Sell";
      TextPrice               =  DoubleToStr(Bid, Digits);
   }
   else
   {
      TextOp                  =  "Buy";
      TextPrice               =  DoubleToStr(Ask, Digits);
   }          
     TextEnter                =  "No";

   if(op                      == 1        ||
      op                      == -1)
   {   
      Enter                   =  "no. 1:";
      TextEnter               =  "Entry_H1";   
   }
   if(IsDemo())
      TextAccount             =  "Demo";
   else  
      TextAccount             =  "Real"; 
   TextTime                   =  TimeToStr(TimeCurrent(), TIME_MINUTES);

   comm                       =  Enter                         +
                                 TextOp                        +
                                 TextAccount                   +
                                 ": "                          +  
                                 TextEnter                     +
                                 "="                           +
                                 TextPrice                     +
                                 " "                           +
                                 TextTime;
//
// ------------ If the market is busy, wait for 5 sec. ----------------------------
   while (!IsTradeAllowed())
   { 
      Sleep(5000);
   }
   RefreshRates();    
// -------------- Lot size -------------------------------------------------
   r_lots                  =  0.1;
   CommonSL                =  NormalizeDouble(CommonSL,  Digits);    
   CommonTP                =  NormalizeDouble(CommonTP,  Digits); 
   slippage                =  NormalizeDouble(slippage,  Digits); 
//
// -------------------- Open a long position ------------------------------------------
   if (op                  <  0)                // Enter the long position - Buy
   {
      RefreshRates();                           // refresh the price 
      price                =  NormalizeDouble(Ask, Digits);             
      Print("");  
      Print("Opening ", comm); 
      Ticket               =  OrderSend(sy,  OP_BUY,
                                 r_lots,        // Number of lots
                                 price,         // Purchase price  
                                 slippage,      // Permissible slippage at the opening
                                 CommonSL,      // Stop loss
                                 CommonTP,      // Take profit
                                 comm,          // Comment
                                 MagicNum,      // Magic number - entry type 
                                 0,             // Contract term
                                 Green);
   }
//
// ------------------- Open a short position -------------------------------------------
   if (op                  >  0)                                    
   {
      RefreshRates();                           // refresh the price 
      price                =  NormalizeDouble(Bid, Digits);   
      Print("");
      Print("Opening ", comm);       
      Ticket               =  OrderSend(  sy,  OP_SELL, 
                                    r_lots,     // Number of lots   
                                    price,      // Market entry price  
                                    slippage,   // Permissible slippage at the opening
                                    CommonSL,   // Stop loss
                                    CommonTP,   // Take profit
                                    comm,       // Comment
                                    MagicNum,   // Magic number - entry type 
                                    0,          // Contract term
                                    Red);       // Arrow color on the chart
   }
//
      
//-----------Processing of the unsuccessful transaction--------------------------------------
   GetError                =  GetLastError();
   if(GetError             >  1)
   {   
      Print("Open_Ord-Error: ",          GetError,
            "-",                          ErrorDescription(GetError));  
      Print("Error position = ",              comm, 
               ", Ask=",                  Ask, 
               ", Bid=",                  Bid, 
               ", r_lots=",               r_lots, 
               ", CommonSL=",             CommonSL, 
               ", CommonTP=",             CommonTP);           
      return(-1);                         
   }
//-----------------------------------------------------------------------------
   return(Ticket);                        // Exit from the user function
}
//
//
// ---------------- Drawing function -----------------------------------------
double  DrawArrow(double  price,   int   lt) 
{
// ----------------- Description of local variables ---------------------------
   double   pr;
 
   pr       =  MathAbs(price);
   pr       =  NormalizeDouble(pr, Digits);
/*   if(ExtDebug)
   {   
      Print(" ");   
      Print("DrawArrow: start= ", price);
   }
*/   
   if(price             >  0.0)
   {
      CountLabel        =  CountLabel  +  1;  
      if(ObjectCreate("Hi"   + CountLabel, OBJ_ARROW, 0, Time[0], pr))
      {
         ObjectSet("Hi"   + CountLabel,   OBJPROP_ARROWCODE,   lt);
         ObjectSet("Hi"   + CountLabel,   OBJPROP_COLOR, Red);
      }   
   }
   if(price             <  0.0)
   {
      CountLabel        =  CountLabel  +  1;  
      if(ObjectCreate("Lo"   + CountLabel, OBJ_ARROW, 0, Time[0], pr))
      {
         ObjectSet("Lo"   + CountLabel,   OBJPROP_ARROWCODE,   lt);
         ObjectSet("Lo"   + CountLabel,   OBJPROP_COLOR, Yellow);
      }   
   }
//   Print("DrawArrow: end. Time=",    TimeHour(Time[0]),   
//          ":",                          TimeMinute(Time[0]),
//          ", price=",                   price);
   return(0.0);
}         
//
// ------------------------- End of Expert Advisor ----------------------------------
// 